@@ -1,13 +1,9 @@
 // Copyright 2020-2021 the Deno authors. All rights reserved. MIT license.
 use super::{Context, LintRule, ProgramRef, DUMMY_NODE};
 use swc_common::Span;
-use swc_ecmascript::ast::CallExpr;
-use swc_ecmascript::ast::Expr;
-use swc_ecmascript::ast::ExprOrSuper;
-use swc_ecmascript::ast::NewExpr;
-use swc_ecmascript::visit::noop_visit_type;
-use swc_ecmascript::visit::Node;
-use swc_ecmascript::visit::Visit;
+use swc_ecmascript::ast::{CallExpr, Expr, ExprOrSuper, Ident, NewExpr};
+use swc_ecmascript::utils::ident::IdentLike;
+use swc_ecmascript::visit::{noop_visit_type, Node, Visit};
 
 pub struct NoObjCalls;
 
@@ -41,6 +37,50 @@ impl LintRule for NoObjCalls {
       ProgramRef::Script(ref s) => visitor.visit_script(s, &DUMMY_NODE),
     }
   }
+
+  fn docs(&self) -> &'static str {
+    r#"Disallows calling built-in global objects like functions
+
+The following built-in objects should not be invoked like functions, even though
+they look like constructors:
+
+- `Math`
+- `JSON`
+- `Reflect`
+- `Atomics`
+
+Calling these as functions would result in runtime errors. This rule statically
+prevents such wrong usage of them.
+
+### Invalid:
+
+```typescript
+const math = Math();
+const newMath = new Math();
+
+const json = JSON();
+const newJSON = new JSON();
+
+const reflect = Reflect();
+const newReflect = new Reflect();
+
+const atomics = Atomics();
+const newAtomics = new Atomics();
+```
+
+### Valid:
+
+```typescript
+const area = (radius: number): number => Math.PI * radius * radius;
+
+const parsed = JSON.parse("{ foo: 42 }");
+
+const x = Reflect.get({ x: 1, y: 2 }, "x");
+
+const first = Atomics.load(foo, 0);
+```
+"#
+  }
 }
 
 struct NoObjCallsVisitor<'c, 'view> {
@@ -52,17 +92,15 @@ impl<'c, 'view> NoObjCallsVisitor<'c, 'view> {
     Self { context }
   }
 
-  fn check_callee(&mut self, callee_name: impl AsRef<str>, span: Span) {
-    let callee_name = callee_name.as_ref();
-    match callee_name {
-      "Math" | "JSON" | "Reflect" | "Atomics" => {
-        self.context.add_diagnostic(
-          span,
-          "no-obj-calls",
-          get_message(callee_name),
-        );
-      }
-      _ => {}
+  fn check_callee(&mut self, callee: &Ident, span: Span) {
+    if matches!(callee.sym.as_ref(), "Math" | "JSON" | "Reflect" | "Atomics")
+      && self.context.scope().var(&callee.to_id()).is_none()
+    {
+      self.context.add_diagnostic(
+        span,
+        "no-obj-calls",
+        get_message(callee.sym.as_ref()),
+      );
     }
   }
 }
@@ -73,14 +111,14 @@ impl<'c, 'view> Visit for NoObjCallsVisitor<'c, 'view> {
   fn visit_call_expr(&mut self, call_expr: &CallExpr, _parent: &dyn Node) {
     if let ExprOrSuper::Expr(expr) = &call_expr.callee {
       if let Expr::Ident(ident) = expr.as_ref() {
-        self.check_callee(&ident.sym, call_expr.span);
+        self.check_callee(ident, call_expr.span);
       }
     }
   }
 
   fn visit_new_expr(&mut self, new_expr: &NewExpr, _parent: &dyn Node) {
     if let Expr::Ident(ident) = &*new_expr.callee {
-      self.check_callee(&ident.sym, new_expr.span);
+      self.check_callee(ident, new_expr.span);
     }
   }
 }
@@ -94,9 +132,29 @@ mod tests {
     assert_lint_ok! {
       NoObjCalls,
       "Math.PI * 2 * 3;",
-      "JSON.parse(\"{}\");",
-      "Reflect.get({ x: 1, y: 2 }, \"x\");",
+      r#"JSON.parse("{}");"#,
+      r#"Reflect.get({ x: 1, y: 2 }, "x");"#,
       "Atomics.load(foo, 0);",
+      r#"
+function f(Math: () => void) {
+  Math();
+}
+      "#,
+      r#"
+function f(JSON: () => void) {
+  JSON();
+}
+      "#,
+      r#"
+function f(Reflect: () => void) {
+  Reflect();
+}
+      "#,
+      r#"
+function f(Atomics: () => void) {
+  Atomics();
+}
+      "#,
     };
   }
 
@@ -112,6 +170,16 @@ mod tests {
       "new Reflect();": [{col: 0, message: get_message("Reflect")}],
       "Atomics();": [{col: 0, message: get_message("Atomics")}],
       "new Atomics();": [{col: 0, message: get_message("Atomics")}],
+      r#"
+function f(Math: () => void) { Math(); }
+const m = Math();
+      "#: [
+        {
+          col: 10,
+          line: 3,
+          message: get_message("Math"),
+        },
+      ],
     }
   }
 }
